from kivy.config import Config
# יש להגדיר את הגופן הדיפולטי לפני כל ייבוא של מודולי Kivy
Config.set('kivy', 'default_font', [
    'NotoSansHebrew-Regular.ttf',
    'NotoSansHebrew-Regular.ttf'
])

import threading
import requests
from kivy.app import App
from kivy.clock import Clock
from kivy.uix.boxlayout import BoxLayout
from kivy.uix.button import Button
from kivy.uix.label import Label
from kivy.uix.scrollview import ScrollView
from kivy.uix.gridlayout import GridLayout
from kivy.uix.textinput import TextInput
from kivy.storage.jsonstore import JsonStore
from bidi.algorithm import get_display  # לטיפול בטקסט RTL

class SearchApp(BoxLayout):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.orientation = 'vertical'
        self.padding = 10
        self.spacing = 10

        # יצירת מחסן קבוע (JSON) לשמירת הגדרות API
        self.store = JsonStore("config.json")
        
        # אתחול משתנים
        self.api_url = ""
        self.token = ""
        
        # אזור להזנת כתובת API
        api_url_layout = BoxLayout(orientation='horizontal', size_hint_y=None, height=40, spacing=5)
        self.api_url_input = TextInput(
            hint_text=get_display("הכנס כתובת API"),
            multiline=False,
            font_name="NotoSansHebrew-Regular.ttf"
        )
        set_api_url_button = Button(
            text=get_display("עדכן כתובת API"),
            size_hint_x=None,
            width=150,
            font_name="NotoSansHebrew-Regular.ttf"
        )
        set_api_url_button.bind(on_press=self.update_api_url)
        api_url_layout.add_widget(self.api_url_input)
        api_url_layout.add_widget(set_api_url_button)
        self.add_widget(api_url_layout)

        # אזור להזנת סיסמת API
        api_token_layout = BoxLayout(orientation='horizontal', size_hint_y=None, height=40, spacing=5)
        self.token_input = TextInput(
            hint_text=get_display("הכנס סיסמת API"),
            multiline=False,
            password=True,
            font_name="NotoSansHebrew-Regular.ttf"
        )
        set_token_button = Button(
            text=get_display("עדכן סיסמה"),
            size_hint_x=None,
            width=150,
            font_name="NotoSansHebrew-Regular.ttf"
        )
        set_token_button.bind(on_press=self.update_token)
        api_token_layout.add_widget(self.token_input)
        api_token_layout.add_widget(set_token_button)
        self.add_widget(api_token_layout)

        # טוען הגדרות קודמות אם קיימות
        if self.store.exists("api_url"):
            self.api_url = self.store.get("api_url")["value"]
            self.api_url_input.text = self.api_url
        if self.store.exists("token"):
            self.token = self.store.get("token")["value"]
            self.token_input.text = self.token

        # שדה להזנת מחרוזת חיפוש
        self.search_input = TextInput(
            hint_text=get_display("הכנס מחרוזת חיפוש"),
            size_hint_y=None,
            height=40,
            multiline=False,
            font_name="NotoSansHebrew-Regular.ttf"
        )
        self.add_widget(self.search_input)

        # כפתור חיפוש
        self.search_button = Button(
            text=get_display("חפש"),
            size_hint_y=None,
            height=40,
            font_name="NotoSansHebrew-Regular.ttf"
        )
        self.search_button.bind(on_press=self.on_search)
        self.add_widget(self.search_button)

        # אזור תצוגת תוצאות בתוך ScrollView
        self.results_layout = GridLayout(cols=1, size_hint_y=None, spacing=5)
        self.results_layout.bind(minimum_height=self.results_layout.setter('height'))
        self.scroll_view = ScrollView(size_hint=(1, 1))
        self.scroll_view.add_widget(self.results_layout)
        self.add_widget(self.scroll_view)

    def update_api_url(self, instance):
        self.api_url = self.api_url_input.text.strip()
        if self.api_url:
            self.store.put("api_url", value=self.api_url)
            self.display_message(get_display("כתובת API עודכנה"))
        else:
            self.display_message(get_display("כתובת API לא חוקית"))

    def update_token(self, instance):
        self.token = self.token_input.text.strip()
        if self.token:
            self.store.put("token", value=self.token)
            self.display_message(get_display("סיסמת API עודכנה"))
        else:
            self.display_message(get_display("סיסמת API לא חוקית"))

    def on_search(self, instance):
        query = self.search_input.text.strip()
        if not query:
            self.display_message(get_display("אנא הכנס מחרוזת חיפוש."))
            return
        if not self.api_url:
            self.display_message(get_display("עדכן כתובת API לפני חיפוש."))
            return
        if not self.token:
            self.display_message(get_display("עדכן סיסמת API לפני חיפוש."))
            return

        # ניקוי תוצאות קודמות והפעלת חיפוש ברקע (ב-thread נפרד)
        self.results_layout.clear_widgets()
        threading.Thread(target=self.search_api, args=(query,), daemon=True).start()

    def search_api(self, query):
        params = {
            "token": self.token,
            "query": query
        }
        try:
            response = requests.get(self.api_url, params=params, timeout=10)
            if response.status_code == 200:
                data = response.json()
                if "error" in data:
                    Clock.schedule_once(lambda dt: self.display_message(
                        get_display("שגיאה: ") + get_display(data["error"])
                    ))
                else:
                    results = data.get("results", [])
                    Clock.schedule_once(lambda dt: self.display_results(results))
            else:
                Clock.schedule_once(lambda dt: self.display_message(
                    get_display(f"שגיאת HTTP: {response.status_code}")
                ))
        except Exception as ex:
            error_message = get_display(f"שגיאה בבקשה: {str(ex)}")
            Clock.schedule_once(lambda dt: self.display_message(error_message))

    def display_results(self, results):
        self.results_layout.clear_widgets()
        if not results:
            self.display_message(get_display("לא נמצאו תוצאות."))
            return

        # הצגת התוצאות, כאשר כל שורה מוצגת עם יישור לימין
        for row in results:
            row_str = ", ".join([get_display(f"{key}: {value}") for key, value in row.items()])
            label = Label(
                text=row_str,
                size_hint_y=None,
                height=30,
                halign='right',
                valign='middle',
                font_name="NotoSansHebrew-Regular.ttf"
            )
            label.bind(size=lambda instance, value: instance.setter('text_size')(instance, value))
            self.results_layout.add_widget(label)

    def display_message(self, message):
        self.results_layout.clear_widgets()
        label = Label(
            text=message,
            size_hint_y=None,
            height=30,
            halign='center',
            valign='middle',
            font_name="NotoSansHebrew-Regular.ttf"
        )
        label.bind(size=lambda instance, value: instance.setter('text_size')(instance, value))
        self.results_layout.add_widget(label)

class MyApp(App):
    def build(self):
        return SearchApp()

if __name__ == '__main__':
    MyApp().run()
